package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.StudentAttendanceViewDto;
import com.ym.mec.biz.dal.dto.TeacherClassStudentDto;
import com.ym.mec.biz.dal.entity.ClassGroupStudentMapper;
import com.ym.mec.biz.dal.entity.StudentRegistration;
import com.ym.mec.biz.dal.enums.ClassGroupStudentStatusEnum;
import com.ym.mec.biz.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface ClassGroupStudentMapperDao extends BaseDAO<Long, ClassGroupStudentMapper> {
    /**
     * 批量插入学生
     *
     * @param classGroupStudentMapperList
     * @return
     */
    int classGroupStudentsInsert(@Param("classGroupStudentMapperList") List<ClassGroupStudentMapper> classGroupStudentMapperList);

    /**
     * 根据班级和学生编号查询
     *
     * @param classGroupId
     * @param userId
     * @return
     */
    ClassGroupStudentMapper query(@Param("classGroupId") int classGroupId, @Param("userId") Integer userId);

    /**
     * 根据乐团编号删除指定学生
     *
     * @param musicGroupId 乐团编号
     * @param userId       学生编号
     * @return
     */
    int deleteStudentByMusicGroupId(@Param("musicGroupId") String musicGroupId, @Param("userId") Integer userId);

    /**
     * 根据班级编号删除关联
     *
     * @param classGroupId
     */
    void deleteByClassId(Integer classGroupId);

    /**
     * @return int
     * @Author: Joburgess
     * @Date: 2019/10/17
     * @params [classGroupId, userId]
     * @describe 删除班级上的指定学生
     */
    int deleteStudentByClassGroupId(@Param("classGroupId") Long classGroupId, @Param("userId") Long userId);

    /**
     * @param classGroupId: 班级编号
     * @return java.lang.String[]
     * @describe 根据班级编号获取班级学生名称列表
     * @author Joburgess
     * @date 2019/10/18
     */
    String[] findStudentNumByClassGroupId(@Param("classGroupId") Long classGroupId);

    /**
     * @param params:
     * @return java.util.List<com.ym.mec.biz.dal.dto.TeacherClassStudentDto>
     * @describe 获取班级学生
     * @author Joburgess
     * @date 2019/10/18
     */
    List<TeacherClassStudentDto> findClassStudent(Map<String, Object> params);

    /**
     * @param params:
     * @return int
     * @describe 统计班级学生数量
     * @author Joburgess
     * @date 2019/10/18
     */
    int findClassStudentNum(Map<String, Object> params);

    /**
     * @param courseScheduleId: 课程编号
     * @return java.util.List<com.ym.mec.biz.dal.entity.StudentAttendance>
     * @describe 获取点名列表
     * @author Joburgess
     * @date 2019/10/19
     */
    List<StudentAttendanceViewDto> findStudentByCourse(Long courseScheduleId);

    /**
     * 查找班级学生对应关系
     *
     * @param userId
     * @param classGroupId
     * @return
     */
    ClassGroupStudentMapper findClassStudentMapperByUserIdAndClassGroupId(@Param("userId") Integer userId, @Param("classGroupId") Integer classGroupId);

    /**
     * @param classGroupIds: 班级编号列表
     * @return java.util.List<com.ym.mec.biz.dal.entity.ClassGroupStudentMapper>
     * @describe 根据班级获取学员
     * @author Joburgess
     * @date 2019/10/24
     */
    List<ClassGroupStudentMapper> findByClassGroups(@Param("classGroupIds") List<Integer> classGroupIds);

    /**
     * @param classGroupId: 班级编号列表
     * @return java.util.List<com.ym.mec.biz.dal.entity.ClassGroupStudentMapper>
     * @describe 根据班级获取学员
     * @author Joburgess
     * @date 2019/10/24
     */
    List<ClassGroupStudentMapper> findByClassGroup(@Param("classGroupId") Integer classGroupId);

    /**
     * 查询班级某状态下所有的学生
     *
     * @param classGroupId
     * @param status
     * @return
     */
    List<StudentRegistration> findClassStudentList(@Param("classGroupId") Integer classGroupId, @Param("status") ClassGroupStudentStatusEnum status);

    /**
     * 查询学生是否已报小班课
     *
     * @param musicGroupId
     * @param userId
     * @param type
     * @return
     */
    List<ClassGroupStudentMapper> findHighClassGroupHasUser(@Param("musicGroupId") String musicGroupId, @Param("userId") Integer userId, @Param("type") ClassGroupTypeEnum type);

    /**
     * @param userId: 用户编号
     * @param date:   日期
     * @param status: 状态
     * @return java.util.List<java.lang.Integer>
     * @describe 获取学生在指定日期对应状态的班级编号
     * @author Joburgess
     * @date 2019/11/20
     */
    List<Integer> findClassGroupIdByUserAndCourseDateAndStatus(@Param("userId") Integer userId,
                                                               @Param("data") Date date,
                                                               @Param("status") String status);

    /**
     * @param classGroupIds: 班级编号列表
     * @return java.util.List<java.util.Map < java.lang.Integer, java.lang.Integer>>
     * @describe 统计班级学生人数
     * @author Joburgess
     * @date 2019/11/21
     */
    List<Map<Integer, Integer>> countClassGroupsStudentNum(@Param("classGroupIds") List<Integer> classGroupIds);

    /**
     * @param classGroupId: 班级编号
     * @return java.util.List<java.util.Map < java.lang.Integer, java.lang.Integer>>
     * @describe 统计班级学生人数
     * @author Joburgess
     * @date 2019/11/21
     */
    Integer countClassGroupStudentNum(@Param("classGroupId") Integer classGroupId);

    /**
     * @describe 统计指定班级的重复人数
     * @author Joburgess
     * @date 2019/11/22
     * @param classGroupId: 班级编号
     * @param userIds: 学生编号
     * @return java.lang.Integer
     */
    Integer countClassGroupExitStudentNum(@Param("classGroupId") Integer classGroupId,
                                          @Param("userIds") List<Integer> userIds);
}
