package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.entity.SysUserRole;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.entity.CourseSchedule.CourseScheduleType;
import com.ym.mec.biz.dal.entity.StudentApplyRefunds.StudentApplyRefundsStatus;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupMember;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class VipGroupServiceImpl extends BaseServiceImpl<Long, VipGroup> implements VipGroupService {
	
	@Autowired
	private VipGroupDao vipGroupDao;
	@Autowired
	private ClassGroupDao classGroupDao;
	@Autowired
	private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;
	@Autowired
	private CourseScheduleService courseScheduleService;
	@Autowired
	private CourseScheduleDao courseScheduleDao;
	@Autowired
	private VipGroupDefaultClassesUnitPriceDao vipGroupDefaultClassesUnitPriceDao;
	@Autowired
	private VipGroupActivityDao vipGroupActivityDao;
	@Autowired
	private VipGroupCategoryDao vipGroupCategoryDao;
	@Autowired
	private MusicGroupDao musicGroupDao;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private TeacherDefaultVipGroupSalaryDao teacherDefaultVipGroupSalaryDao;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
	private StudentApplyRefundsDao studentApplyRefundsDao;
	@Autowired
	private SysUserCashAccountService sysUserCashAccountService;
    @Autowired
	private SysUserCashAccountDetailDao sysUserCashAccountDetailDao;
    @Autowired
	private ClassGroupTeacherSalaryDao classGroupTeacherSalaryDao;
    @Autowired
	private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
    @Autowired
	private CourseScheduleTeacherSalaryDao courseScheduleTeacherSalaryDao;
    @Autowired
	private ImFeignService imFeignService;
    @Autowired
	private SubjectDao subjectDao;
    @Autowired
	private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;
    @Autowired
	private CourseScheduleStudentPaymentService courseScheduleStudentPaymentService;
	@Autowired
	private PayService payService;
	@Autowired
	private ClassGroupStudentMapperDao classGroupStudentMapperDao;
	@Autowired
	private CourseScheduleRewardsRulesService courseScheduleRewardsRulesService;
	@Autowired
	private SysConfigDao sysConfigDao;
	@Autowired
	private SysUserCashAccountDetailService sysUserCashAccountDetailService;
	@Autowired
	private SysMessageService sysMessageService;
	@Autowired
	private VipGroupDefaultClassesCycleDao vipGroupDefaultClassesCycleDao;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private TeacherAttendanceDao teacherAttendanceDao;
	@Autowired
	private ClassGroupService classGroupService;
	@Autowired
	private StudentPaymentOrderService studentPaymentOrderService;

	@Override
	public BaseDAO<Long, VipGroup> getDAO() {
		return vipGroupDao;
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void createVipGroup(VipGroupApplyDto vipGroup,Integer systemId) {

		if (Objects.isNull(vipGroup.getVipGroupApplyBaseInfo().getUserId())){
			throw new BizException("请选择指导老师");
		}

		if(Objects.isNull(vipGroup.getVipGroupApplyBaseInfo().getTeacherSchoolId())){
			throw new BizException("请设置教学点");
		}

	    if(vipGroup.getCourseSchedules().size()!=(vipGroup.getVipGroupApplyBaseInfo().getOfflineClassesNum()+vipGroup.getVipGroupApplyBaseInfo().getOnlineClassesNum())){
	        throw new BizException("建课失败，当前课程存在未排课课程，请调整相关设置");
        }

		Map<TeachModeEnum, List<CourseSchedule>> courseScheduleGroup = null;
		try {
			courseScheduleGroup = vipGroup.getCourseSchedules().stream().collect(Collectors.groupingBy(CourseSchedule::getTeachMode));
		} catch (Exception e) {
			throw new BizException("教学模式错误");
		}

		if((Objects.isNull(courseScheduleGroup.get(TeachModeEnum.OFFLINE))&&vipGroup.getVipGroupApplyBaseInfo().getOfflineClassesNum()!=0)
				||(!Objects.isNull(courseScheduleGroup.get(TeachModeEnum.OFFLINE))&&(courseScheduleGroup.get(TeachModeEnum.OFFLINE).size()<vipGroup.getVipGroupApplyBaseInfo().getOfflineClassesNum()))){
	        throw new BizException("线下课课时数量安排有误");
        }

        if((Objects.isNull(courseScheduleGroup.get(TeachModeEnum.ONLINE))&&vipGroup.getVipGroupApplyBaseInfo().getOnlineClassesNum()!=0)
				||(!Objects.isNull(courseScheduleGroup.get(TeachModeEnum.ONLINE))&&(courseScheduleGroup.get(TeachModeEnum.ONLINE).size()<vipGroup.getVipGroupApplyBaseInfo().getOnlineClassesNum()))){
            throw new BizException("线上课课时数量安排有误");
        }

		SysUser sysUser = sysUserFeignService.queryUserById(vipGroup.getVipGroupApplyBaseInfo().getUserId());

        if(Objects.isNull(sysUser)){
        	throw new BizException("该用户不存在");
		}

        String studentIds=vipGroup.getStudentIds();
		List<String> studentIdList=new ArrayList<>();

        if(StringUtils.isNotBlank(studentIds)){
			studentIdList = Arrays.asList(studentIds.split(","));
		}

        Date now=new Date();

		VipGroupApplyBaseInfoDto vipGroupApplyBaseInfoDto=vipGroup.getVipGroupApplyBaseInfo();

		//获取活动信息
		VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroup.getVipGroupApplyBaseInfo().getVipGroupActivityId().intValue());
		Integer totalClassTimes=vipGroupApplyBaseInfoDto.getOnlineClassesNum()+vipGroupApplyBaseInfoDto.getOfflineClassesNum();
		//获取第一节课
		CourseSchedule firstCourseSchedule = vipGroup.getCourseSchedules().stream().min(Comparator.comparing(CourseSchedule::getStartClassTime)).get();
		//获取最后一节课
		CourseSchedule latestCourseSchedule = vipGroup.getCourseSchedules().stream().max(Comparator.comparing(CourseSchedule::getEndClassTime)).get();
		//判断课程安排是否超出范围
		if(Objects.nonNull(vipGroupActivity.getCoursesEndTime())||Objects.nonNull(vipGroupActivity.getCoursesStartTime())){
			if(latestCourseSchedule.getEndClassTime().after(vipGroupActivity.getCoursesEndTime())
					||firstCourseSchedule.getStartClassTime().before(vipGroupActivity.getCoursesStartTime())){
				throw new BizException("课时安排时间超出范围!");
			}
		}else{
			VipGroupDefaultClassesCycle vipGroupDefaultClassesCycle = vipGroupDefaultClassesCycleDao.findByOrganAndClassTimes(Long.parseLong(sysUser.getOrganId()), totalClassTimes);
			int month = DateUtil.monthsBetween(firstCourseSchedule.getStartClassTime(), latestCourseSchedule.getEndClassTime());
			if(month>vipGroupDefaultClassesCycle.getMonth()){
				throw new BizException("课时安排时间超出范围!");
			}
		}

		//生成vip课信息
		List<String> bySubIds = subjectDao.findBySubIds(vipGroupApplyBaseInfoDto.getSubjectIdList());
		StringBuffer className=new StringBuffer(StringUtils.join(bySubIds,","));
		className.append("•");
		className.append(vipGroupApplyBaseInfoDto.getName());
		vipGroupApplyBaseInfoDto.setName(className.toString());

		//计算课程相关费用信息
		Map<String, BigDecimal> costInfo = countVipGroupPredictFee(vipGroupApplyBaseInfoDto,
				vipGroupApplyBaseInfoDto.getUserId(),
				vipGroupApplyBaseInfoDto.getOnlineClassesUnitPrice(),
				vipGroupApplyBaseInfoDto.getOfflineClassesUnitPrice(),
				true,
				true);

//		BigDecimal offlineClassNum=new BigDecimal(vipGroupApplyBaseInfoDto.getOfflineClassesNum());
//		BigDecimal onlineClassNum=new BigDecimal(vipGroupApplyBaseInfoDto.getOnlineClassesNum());
//
//		if(vipGroupActivity.getType().equals(VipGroupActivityTypeEnum.GIVE_CLASS)){
//			if(vipGroupApplyBaseInfoDto.getGiveTeachMode()==TeachModeEnum.OFFLINE){
//				if(new BigDecimal(totalClassTimes).compareTo(new BigDecimal(vipGroupActivity.getAttribute1()))>-1){
//					offlineClassNum=offlineClassNum.subtract(new BigDecimal(vipGroupActivity.getAttribute2()));
//				}
//			}else if(vipGroupApplyBaseInfoDto.getGiveTeachMode()==TeachModeEnum.ONLINE){
//				if(new BigDecimal(totalClassTimes).compareTo(new BigDecimal(vipGroupActivity.getAttribute1()))>-1){
//					onlineClassNum=onlineClassNum.subtract(new BigDecimal(vipGroupActivity.getAttribute2()));
//				}
//			}else{
//				throw new BizException("请指定赠送课程类型!");
//			}
//		}

//		BigDecimal tempFee = costInfo.get("totalPrice").multiply(new BigDecimal(0.6));
//		BigDecimal totalSalary = vipGroupApplyBaseInfoDto.getOfflineTeacherSalary().multiply(offlineClassNum)
//				.add(vipGroupApplyBaseInfoDto.getOnlineTeacherSalary().multiply(onlineClassNum));
//		if(totalSalary.compareTo(tempFee)>0){
//			throw new BizException("教师总课酬不能高于课程总价60%");
//		}

		//如果默认课酬与实际课酬不匹配则需要审批
		if(costInfo.get("offlineTeacherSalary").compareTo(vipGroupApplyBaseInfoDto.getOfflineTeacherSalary())!=0||
			costInfo.get("onlineTeacherSalary").compareTo(vipGroupApplyBaseInfoDto.getOnlineTeacherSalary())!=0
			&&StringUtils.isBlank(studentIds)){
			vipGroupApplyBaseInfoDto.setAuditStatus(AuditStatusEnum.ING);
		}else{
			vipGroupApplyBaseInfoDto.setAuditStatus(AuditStatusEnum.PASS);
		}

		vipGroupApplyBaseInfoDto.setTotalPrice(costInfo.get("totalPrice"));
		if(StringUtils.isNotBlank(studentIds)){
			vipGroupApplyBaseInfoDto.setStatus(VipGroupStatusEnum.PROGRESS);
		}else{
			vipGroupApplyBaseInfoDto.setStatus(VipGroupStatusEnum.APPLYING);
		}
		vipGroupApplyBaseInfoDto.setOrganId(Integer.parseInt(sysUser.getOrganId()));
		//开课时间为排课的第一节课的开始时间
		vipGroupApplyBaseInfoDto.setCourseStartDate(firstCourseSchedule.getStartClassTime());
		//课程结束时间为排课的最后一节课的结束时间
		vipGroupApplyBaseInfoDto.setCoursesExpireDate(latestCourseSchedule.getEndClassTime());
		vipGroupDao.insert(vipGroupApplyBaseInfoDto);

		//获取vip课类型信息
		VipGroupCategory vipGroupCategory = vipGroupCategoryDao.get(vipGroup.getVipGroupApplyBaseInfo().getVipGroupCategoryId().intValue());

		//创建班级信息
		ClassGroup classGroup=new ClassGroup();
		classGroup.setSubjectIdList(vipGroupApplyBaseInfoDto.getSubjectIdList());
		classGroup.setExpectStudentNum(vipGroupCategory.getStudentNum());
		if(StringUtils.isNotBlank(studentIds)){
			classGroup.setStudentNum(studentIdList.size());
		}
		classGroup.setName(vipGroupApplyBaseInfoDto.getName());
		classGroup.setExpectStudentNum(vipGroupApplyBaseInfoDto.getStudentNum());
		classGroup.setTotalClassTimes(totalClassTimes);
		classGroup.setType(ClassGroupTypeEnum.VIP);
		classGroup.setDelFlag(YesOrNoEnum.YES);
		classGroup.setGroupType(GroupType.VIP);
		classGroup.setMusicGroupId(vipGroupApplyBaseInfoDto.getId().toString());
		classGroup.setCreateTime(now);
		classGroup.setUpdateTime(now);
		classGroupDao.insert(classGroup);

		//创建班级老师关联记录
		ClassGroupTeacherMapper classGroupTeacherMapper=new ClassGroupTeacherMapper();
		classGroupTeacherMapper.setMusicGroupId(vipGroupApplyBaseInfoDto.getId().toString());
		classGroupTeacherMapper.setClassGroupId(classGroup.getId());
		classGroupTeacherMapper.setTeacherRole(TeachTypeEnum.BISHOP);
		classGroupTeacherMapper.setUserId(vipGroupApplyBaseInfoDto.getUserId());
		classGroupTeacherMapper.setGroupType(GroupType.VIP);
		classGroupTeacherMapper.setCreateTime(now);
		classGroupTeacherMapper.setUpdateTime(now);
		classGroupTeacherMapperDao.insert(classGroupTeacherMapper);

		//创建班级与老师课酬记录
		ClassGroupTeacherSalary classGroupTeacherSalary=new ClassGroupTeacherSalary();
		classGroupTeacherSalary.setMusicGroupId(vipGroupApplyBaseInfoDto.getId().toString());
		classGroupTeacherSalary.setClassGroupId(classGroup.getId());
		classGroupTeacherSalary.setTeacherRole(TeachTypeEnum.BISHOP);
		classGroupTeacherSalary.setUserId(vipGroupApplyBaseInfoDto.getUserId());
		classGroupTeacherSalary.setSalary(vipGroupApplyBaseInfoDto.getOfflineTeacherSalary());
		classGroupTeacherSalary.setOnlineClassesSalary(vipGroupApplyBaseInfoDto.getOnlineTeacherSalary());
		classGroupTeacherSalary.setGroupType(GroupType.VIP);
		classGroupTeacherSalary.setCreateTime(now);
		classGroupTeacherSalary.setUpdateTime(now);
		classGroupTeacherSalaryDao.insert(classGroupTeacherSalary);

		//课程信息调整
		vipGroup.getCourseSchedules().forEach(courseSchedule -> {
			courseSchedule.setGroupType(GroupType.VIP);
			courseSchedule.setMusicGroupId(vipGroupApplyBaseInfoDto.getId().toString());
		    courseSchedule.setSchoolId(vipGroup.getVipGroupApplyBaseInfo().getTeacherSchoolId());
		    courseSchedule.setTeacherId(vipGroupApplyBaseInfoDto.getUserId());
			courseSchedule.setActualTeacherId(vipGroupApplyBaseInfoDto.getUserId());
			courseSchedule.setStatus(CourseStatusEnum.NOT_START);
		    courseSchedule.setType(CourseSchedule.CourseScheduleType.VIP);
			courseSchedule.setClassGroupId(classGroup.getId());
		});
		courseScheduleService.checkNewCourseSchedules(vipGroup.getCourseSchedules(),false);
		vipGroupApplyBaseInfoDto.setCourseSchedulesJson(JSON.toJSONString(vipGroup.getCourseSchedules()));
		vipGroupDao.update(vipGroupApplyBaseInfoDto);

		if(StringUtils.isNotBlank(studentIds)){
			List<Integer> collect = studentIdList.stream().mapToInt(Integer::parseInt).boxed().collect(Collectors.toList());
			classGroupService.addStudentIntoClassGroup(vipGroupApplyBaseInfoDto.getId().toString(),classGroup.getId(),collect);
			createVipGroupCourseScheInfo(vipGroupApplyBaseInfoDto.getId(),classGroup);
		}

		Set<Integer> roleIds = new HashSet<>(1);
		roleIds.add(SysUserRole.OPERATION_EXECUTIVE);
		Map<String,Long> memo = new HashMap<>(1);
		memo.put("vipGroupId",vipGroupApplyBaseInfoDto.getId());
		Set<Integer> userIds = musicGroupDao.queryUserIdByRoleId(roleIds);
		if (systemId != null){
			SysUser systemUser = sysUserFeignService.queryUserById(systemId);
			sysMessageService.batchSeoMessage(userIds,MessageTypeEnum.BACKSTAGE_VIP_COURSE_APPLY, JSONObject.toJSONString(memo),systemUser.getUsername(),sysUser.getUsername());
		}else {
			sysMessageService.batchSeoMessage(userIds,MessageTypeEnum.BACKSTAGE_TEACHER_APPLY_VIP, JSONObject.toJSONString(memo),sysUser.getUsername());
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateVipGroup(VipGroupApplyDto vipGroupApplyInfo) {
		VipGroupApplyBaseInfoDto vipGroupUpdateInfo = vipGroupApplyInfo.getVipGroupApplyBaseInfo();
		//更新vip课
		updateVipGroupBaseInfo(vipGroupUpdateInfo);

		//课程调整
		List<CourseSchedule> courseSchedules = vipGroupApplyInfo.getCourseSchedules();
		if(CollectionUtils.isEmpty(courseSchedules)){
			return;
		}

		//只需要调整课程信息的课程列表
		List<CourseSchedule> updateCourseSchedules = courseSchedules
				.stream()
				.filter(courseSchedule -> Objects.nonNull(courseSchedule.getId()))
				.collect(Collectors.toList());
		List<Long> updateCourseScheduleIds = updateCourseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toList());

		//新增的课程列表
		List<CourseSchedule> newCourseSchedules = courseSchedules
				.stream()
				.filter(courseSchedule -> Objects.nonNull(courseSchedule.getId()))
				.collect(Collectors.toList());

		//指定VIP课原有的课程列表
		List<CourseSchedule> vipGroupCourseSchedules = courseScheduleDao.findVipGroupCourseSchedules(vipGroupUpdateInfo.getId());

		if(!CollectionUtils.isEmpty(updateCourseSchedules)){
			//需要删除的课程编号列表
			List<Long> deleteCourseScheduleIds = vipGroupCourseSchedules.stream()
					.filter(courseSchedule -> !updateCourseScheduleIds.contains(courseSchedule.getId()))
					.map(CourseSchedule::getId)
					.collect(Collectors.toList());
			//删除对应的课程及相关信息
			courseScheduleService.deleteCourseSchedules(deleteCourseScheduleIds);

		}

		//检测课程信息是否存在冲突
		courseScheduleService.checkNewCourseSchedules(courseSchedules,false);
		//只需要更新的课程走课程调整流程
		courseScheduleService.classStartDateAdjust(updateCourseSchedules);

		//新增的课程需要重新创建相关信息
		courseScheduleService.batchAddCourseSchedule(newCourseSchedules);
		//创建老师单节课课酬信息
		courseScheduleTeacherSalaryService.createCourseScheduleTeacherVipSalary(vipGroupUpdateInfo,
				newCourseSchedules,
				vipGroupUpdateInfo.getOnlineTeacherSalary(),
				vipGroupUpdateInfo.getOfflineTeacherSalary());

	}

	@Override
	public void vipApplyAudit(Long vipGroupId, AuditStatusEnum auditStatus, String reason) {
		if(Objects.isNull(vipGroupId)||Objects.isNull(auditStatus)){
			throw new BizException("请指定vip课及审核结果");
		}
		VipGroup vipGroup = vipGroupDao.get(vipGroupId);
		if(Objects.isNull(vipGroup)){
			throw new BizException("vip课不存在");
		}
		if(vipGroup.getAuditStatus()!=AuditStatusEnum.ING){
			throw new BizException("该课程已被审核");
		}
		if(auditStatus==AuditStatusEnum.REJECT&&StringUtils.isBlank(reason)){
			throw new BizException("请填写拒绝原因");
		}
		/*if(vipGroup.getAuditStatus() == AuditStatusEnum.PASS){
			ImGroupMember[] imGroupMembers = new ImGroupMember[]{new ImGroupMember(vipGroup.getUserId().toString())};
			// 创建群组
			imFeignService.groupCreate(new ImGroupModel(vipGroupId.toString(), imGroupMembers, vipGroup.getName()));
		}*/
		vipGroup.setAuditStatus(auditStatus);
		vipGroup.setStopReason(reason);
		vipGroupDao.update(vipGroup);
		//给老师推送审批短信
		Teacher teacher = teacherDao.get(vipGroup.getUserId());
		if(teacher != null){
			Map<Integer,String> map = new HashMap<>(1);
			map.put(vipGroup.getUserId(),vipGroup.getUserId().toString());
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,MessageTypeEnum.TEACHER_SMS_VIP_COURSE_APPLY_RESULT,
					map,null,0,"7",vipGroup.getName(),vipGroup.getAuditStatus().getMsg());
			map.put(vipGroup.getUserId(),teacher.getPhone());
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI,MessageTypeEnum.TEACHER_SMS_VIP_COURSE_APPLY_RESULT,
					map,null,0,"7",vipGroup.getName(),vipGroup.getAuditStatus().getMsg());
		}
	}

	@Override
	public PageInfo findStudentVipGroupList(StudentVipGroupQueryInfo queryInfo) {
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = vipGroupDao.countVipGorups(params);
		if (count > 0) {

			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupDao.findVipGroups(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo findVipGroups(VipGroupQueryInfo queryInfo) {
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<VipGroup> dataList = null;
		int count = vipGroupDao.countVipGroupByOrgan(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupDao.findAllByOrgan(params);
			List<String> vipGroupIds = dataList.stream().map(vipGroup -> String.valueOf(vipGroup.getId())).distinct().collect(Collectors.toList());
			List<Map<Long, Integer>> vipGroupOverCourses = vipGroupDao.countVipGroupOverCourse(vipGroupIds);
			Map<Long,Integer> vipGroupOverCourseMaps = MapUtil.convertIntegerMap(vipGroupOverCourses);

			List<ClassGroup> classGroupsByVipGroups = classGroupDao
					.findByMusicGroupsAndType(vipGroupIds.stream().map(vipGroupId->String.valueOf(vipGroupId)).collect(Collectors.toList()),
							GroupType.VIP.getCode());

			Map<String, ClassGroup> vipGroupClassGroupMap = classGroupsByVipGroups.stream().collect(Collectors.toMap(ClassGroup::getMusicGroupId, classGroup -> classGroup));
			dataList.forEach(vipGroup -> {
				Integer overCourses = vipGroupOverCourseMaps.get(vipGroup.getId());
				if(Objects.isNull(overCourses)){
					overCourses=0;
				}
				Integer totalClassTimes=vipGroup.getTotalClassTimes();
				if(Objects.isNull(totalClassTimes)){
					vipGroup.setMonthConsumeRate(0.0);
				}else{
					if(totalClassTimes<=0){
						vipGroup.setMonthConsumeRate((double) 0);
					}else{
						BigDecimal overCoursesOfMonth=new BigDecimal(overCourses),allClassTimes=new BigDecimal(totalClassTimes);

						Double monthConsumeRate=overCoursesOfMonth.divide(allClassTimes,2,BigDecimal.ROUND_HALF_UP).doubleValue();
						vipGroup.setMonthConsumeRate(monthConsumeRate);
					}

				}
				ClassGroup classGroup = vipGroupClassGroupMap.get(vipGroup.getId().toString());
				if(Objects.nonNull(classGroup)){
					vipGroup.setStudentNum(classGroup.getStudentNum());
					vipGroup.setTotalClassTimes(classGroup.getTotalClassTimes());
					vipGroup.setCurrentClassTimes(classGroup.getCurrentClassTimes());
				}

			});
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public Map<String, Object> findVipGroupTeachingRecord(VipGroupTeachingRecordQueryInfo queryInfo) {
		if(Objects.isNull(queryInfo.getVipGroupId())){
			throw new BizException("请指定vip课程!");
		}
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = vipGroupDao.countTeachingRecord(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupDao.findTeachingRecord(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		TeachingRecordBaseInfo classTimes = vipGroupDao.getClassTimes(queryInfo.getVipGroupId());
		if(!Objects.isNull(classTimes)){
			classTimes.setIsSalaryNum(vipGroupDao.countIsSalaryNum(queryInfo.getVipGroupId()));
			classTimes.setComplaintsNum(vipGroupDao.countComplaintsNum(queryInfo.getVipGroupId()));
		}
		Map<String,Object> result=new HashMap<>();
		result.put("pageInfo",pageInfo);
		result.put("baseInfo",classTimes);
		return result;
	}

	@Override
	public TeachingRecordBaseInfo findTeachingRecordBaseInfo(Long vipGroupId) {
        TeachingRecordBaseInfo classTimes = vipGroupDao.getClassTimes(vipGroupId);
        classTimes.setIsSalaryNum(vipGroupDao.countIsSalaryNum(vipGroupId));
        classTimes.setComplaintsNum(vipGroupDao.countComplaintsNum(vipGroupId));
        return classTimes;
	}

	@Override
	public PageInfo findVipGroupStudents(VipGroupQueryInfo queryInfo) {
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = vipGroupDao.countVipGroupStudents(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupDao.findVipGroupStudents(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo findHaveCourseBalanceStudents(VipGroupQueryInfo queryInfo) {
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = vipGroupDao.countHaveCourseBalanceStudentNum(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupDao.findHaveCourseBalanceStudents(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public VipGroupManageDetailDto getVipGroupDetail(Long vipGroupId) {
		if(Objects.isNull(vipGroupId)){
			throw new BizException("请指定vip课程");
		}
		VipGroupManageDetailDto vipGroupBaseInfo = vipGroupDao.getVipGroupBaseInfo(vipGroupId);
		vipGroupBaseInfo.setSubjectName(StringUtils.join(subjectDao.findBySubIds(vipGroupBaseInfo.getSubjectIdList()),","));
		ClassGroupTeacherSalary byVipGoupAndTeacher = classGroupTeacherSalaryDao.findByVipGoupAndTeacher(vipGroupId.intValue(), vipGroupBaseInfo.getUserId());
		if(Objects.nonNull(byVipGoupAndTeacher)){
			vipGroupBaseInfo.setOnlineTeacherSalary(byVipGoupAndTeacher.getOnlineClassesSalary());
			vipGroupBaseInfo.setOfflineTeacherSalary(byVipGoupAndTeacher.getSalary());
		}
		return vipGroupBaseInfo;
	}

	@Override
	public PageInfo findVipGroupAttendances(VipGroupAttendanceQueryInfo queryInfo) {
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = vipGroupDao.countVipGroupAttendance(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupDao.findVipGroupAttendances(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public StudentVipGroupDetailDto getVipGroupShowDetail(Long vipGroupId) {
		if(Objects.isNull(vipGroupId)){
			throw new BizException("请指定vip课程");
		}
		VipGroup vipGroup = vipGroupDao.get(vipGroupId);
		if(Objects.isNull(vipGroup)){
			throw new BizException("未找到指定课程");
		}
		StudentVipGroupDetailDto vipGroupDetail = vipGroupDao.getVipGroupDetail(vipGroupId);
		if(StringUtils.isNotEmpty(vipGroup.getCourseSchedulesJson())){
			vipGroupDetail.setCourseSchedules(JSON.parseArray(vipGroup.getCourseSchedulesJson(),CourseSchedule.class));
			List<CourseSchedule> collect = vipGroupDetail.getCourseSchedules().stream()
					.sorted(Comparator.comparing(CourseSchedule::getStartClassTime))
					.collect(Collectors.toList());
			vipGroupDetail.setCourseSchedules(collect);
		}

        vipGroupDetail.setNumberOfClasses(vipGroupDao.countTeacherVipGroups(vipGroupDetail.getTeacherId().longValue()));
        return vipGroupDetail;
	}

	@Override
	public VipGroupCostCountDto vipGroupCostCount(VipGroupCostCountParamsDto vipGroupCostCountParamsDto) {
        SysUser user = sysUserFeignService.queryUserInfo();
        TeacherDefaultVipGroupSalary teacherDefaultVipGroupSalary = teacherDefaultVipGroupSalaryDao.findByTeacherAndCategory(user.getId(), vipGroupCostCountParamsDto.getVipGroupCategoryId());
        VipGroupDefaultClassesUnitPrice vipGroupDefaultClassesUnitPrice = vipGroupDefaultClassesUnitPriceDao.get(vipGroupCostCountParamsDto.getVipGroupDefaultClassesUnitPriceId().intValue());
		VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroupCostCountParamsDto.getVipGroupActivityId().intValue());
        VipGroupSalarySettlementDto vipGroupSalarySettlementDto = JSON.parseObject(vipGroupActivity.getSalarySettlementJson(), VipGroupSalarySettlementDto.class);
        BigDecimal offlineClassNum=new BigDecimal(vipGroupCostCountParamsDto.getOfflineClassesNum());
        BigDecimal onlineClassNum=new BigDecimal(vipGroupCostCountParamsDto.getOnlineClassesNum());


        //教师课酬
        BigDecimal teacherOnlineSalary,teacherOfflineSalary,teacherSalary;
        switch (vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSalarySettlementType()){
            case TEACHER_DEFAULT:
                teacherOnlineSalary=teacherDefaultVipGroupSalary.getOnlineClassesSalary().multiply(onlineClassNum);
                break;
            case RATIO_DISCOUNT:
                teacherOnlineSalary=vipGroupDefaultClassesUnitPrice.getOnlineClassesUnitPrice().multiply(vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSettlementValue()).multiply(onlineClassNum);
                break;
            case FIXED_SALARY:
                teacherOnlineSalary=vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSettlementValue().multiply(onlineClassNum);
                break;
             default:
                 throw new BizException("未指定课酬结算标准!");
        }
        switch (vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSalarySettlementType()){
            case TEACHER_DEFAULT:
                teacherOfflineSalary=teacherDefaultVipGroupSalary.getOfflineClassesSalary().multiply(offlineClassNum);
                break;
            case RATIO_DISCOUNT:
                teacherOfflineSalary=vipGroupDefaultClassesUnitPrice.getOfflineClassesUnitPrice().multiply(vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSettlementValue()).multiply(offlineClassNum);
                break;
            case FIXED_SALARY:
                teacherOfflineSalary=vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSettlementValue().multiply(offlineClassNum);
                break;
            default:
                throw new BizException("未指定课酬结算标准!");
        }
        teacherSalary=teacherOnlineSalary.add(teacherOfflineSalary);
        return new VipGroupCostCountDto(null,teacherSalary);
    }

    @Override
    public <K extends VipGroup> Map<String,BigDecimal> countVipGroupPredictFee(K vipGroup,
																			   Integer teacherId,
																			   BigDecimal onlineClassesUnitPrice,
																			   BigDecimal offlineClassesUnitPrice,
																			   boolean computeTotalPrice,
																			   boolean useDefaultTeacherSalary){

		if(Objects.isNull(teacherId)){
			throw new BizException("请指定教师");
		}

		if(Objects.isNull(vipGroup.getVipGroupCategoryId())){
			throw new BizException("请指定课程形式");
		}

		if(Objects.isNull(vipGroup.getSingleClassMinutes())){
			throw new BizException("请指定课程时长");
		}

		VipGroupCategory vipGroupCategory = vipGroupCategoryDao.get(vipGroup.getVipGroupCategoryId().intValue());
		if(Objects.isNull(vipGroupCategory)){
			throw new BizException("未找到课程形式");
		}

		TeacherDefaultVipGroupSalary teacherDefaultVipGroupSalary = teacherDefaultVipGroupSalaryDao.findByTeacherAndCategory(teacherId,
				vipGroup.getVipGroupCategoryId());

		if(Objects.isNull(vipGroup.getVipGroupActivityId())){
			throw new BizException("请指定活动方案");
		}

		VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroup.getVipGroupActivityId().intValue());

		if(Objects.isNull(vipGroupActivity)){
			throw new BizException("指定的活动方案不存在");
		}

		VipGroupSalarySettlementDto vipGroupSalarySettlementDto = JSON.parseObject(vipGroupActivity.getSalarySettlementJson(), VipGroupSalarySettlementDto.class);

		if(Objects.isNull(vipGroupSalarySettlementDto)){
			throw new BizException("课酬结算方案错误");
		}

		Map<String,BigDecimal> results=new HashMap<>(1);

		BigDecimal teacherSalaryTimeUnit=new BigDecimal(sysConfigDao.findConfigValue(SysConfigService.TEACHER_SALARY_TIME_UNIT));
		//课程时长与结算单位时长占比
		BigDecimal classTimeDuty=new BigDecimal(vipGroup.getSingleClassMinutes()).divide(teacherSalaryTimeUnit,2,BigDecimal.ROUND_DOWN);
		BigDecimal offlineClassNum=new BigDecimal(vipGroup.getOfflineClassesNum());
		BigDecimal onlineClassNum=new BigDecimal(vipGroup.getOnlineClassesNum());
		onlineClassesUnitPrice=Objects.isNull(onlineClassesUnitPrice)?new BigDecimal(0):onlineClassesUnitPrice;
		BigDecimal onlineVipGroupCharge = onlineClassesUnitPrice.multiply(onlineClassNum);
		offlineClassesUnitPrice=Objects.isNull(offlineClassesUnitPrice)?new BigDecimal(0):offlineClassesUnitPrice;
		BigDecimal offlineVipGroupCharge = offlineClassesUnitPrice.multiply(offlineClassNum);
		BigDecimal totalClassNum=offlineClassNum.add(onlineClassNum);
		BigDecimal teacherOnlineSalary=null,teacherOfflineSalary=null;

		//教师课酬线上单课酬计算
		if(Objects.nonNull(vipGroupSalarySettlementDto.getOnlineSalarySettlement())){
            if(vipGroup instanceof VipGroupApplyBaseInfoDto&&!useDefaultTeacherSalary
                    &&vipGroupActivity.getSalaryReadonlyFlag().equals("1")){
                teacherOnlineSalary=((VipGroupApplyBaseInfoDto) vipGroup).getOnlineTeacherSalary();
                results.put("onlineTeacherSalary",teacherOnlineSalary);
            }
            if(Objects.isNull(teacherOnlineSalary)){
                switch (vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSalarySettlementType()){
                    case TEACHER_DEFAULT:
                        if(Objects.isNull(teacherDefaultVipGroupSalary)||Objects.isNull(teacherDefaultVipGroupSalary.getOnlineClassesSalary())){
                            teacherOnlineSalary=new BigDecimal(0);
                        }else{
                            teacherOnlineSalary=teacherDefaultVipGroupSalary.getOnlineClassesSalary();
                        }
//                        teacherOnlineSalary=teacherOnlineSalary.multiply(classTimeDuty);
                        results.put("onlineTeacherSalary",teacherOnlineSalary);
                        break;
                    case RATIO_DISCOUNT:
                        results.put("onlineTeacherSalary",onlineClassesUnitPrice.multiply((vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSettlementValue())).divide(new BigDecimal(100)).setScale(2,BigDecimal.ROUND_HALF_UP));
                        break;
                    case FIXED_SALARY:
                        results.put("onlineTeacherSalary",vipGroupSalarySettlementDto.getOnlineSalarySettlement().getSettlementValue());
                        break;
                    default:
                        throw new BizException("未指定课酬结算标准!");
                }
            }
		}

		//教师线下单课酬计算
		if(Objects.nonNull(vipGroupSalarySettlementDto.getOfflineSalarySettlement())){
            if(vipGroup instanceof VipGroupApplyBaseInfoDto&&!useDefaultTeacherSalary
                    &&vipGroupActivity.getSalaryReadonlyFlag().equals("1")){
                teacherOfflineSalary=((VipGroupApplyBaseInfoDto) vipGroup).getOfflineTeacherSalary();
                results.put("offlineTeacherSalary",teacherOfflineSalary);
            }
            if(Objects.isNull(teacherOfflineSalary)){
                switch (vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSalarySettlementType()){
                    case TEACHER_DEFAULT:
                        if(Objects.isNull(teacherDefaultVipGroupSalary)||Objects.isNull(teacherDefaultVipGroupSalary.getOfflineClassesSalary())){
                            teacherOfflineSalary=new BigDecimal(0);
                        }else{
                            teacherOfflineSalary=teacherDefaultVipGroupSalary.getOfflineClassesSalary();
                        }
//                        teacherOfflineSalary=teacherOfflineSalary.multiply(classTimeDuty);
                        results.put("offlineTeacherSalary",teacherOfflineSalary);
                        break;
                    case RATIO_DISCOUNT:
                        results.put("offlineTeacherSalary",offlineClassesUnitPrice.multiply(vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSettlementValue()).divide(new BigDecimal(100)).setScale(2,BigDecimal.ROUND_HALF_UP));
                        break;
                    case FIXED_SALARY:
                        results.put("offlineTeacherSalary",vipGroupSalarySettlementDto.getOfflineSalarySettlement().getSettlementValue());
                        break;
                    default:
                        throw new BizException("未指定课酬结算标准!");
                }
            }
		}

		if(computeTotalPrice){
			//课程购买费用计算
			BigDecimal totalPrice;
			switch (vipGroupActivity.getType()){
				case BASE_ACTIVITY:
					totalPrice=onlineVipGroupCharge.add(offlineVipGroupCharge);
					break;
				case DISCOUNT:
					BigDecimal discount=new BigDecimal(vipGroupActivity.getAttribute1());
					totalPrice=onlineVipGroupCharge.add(offlineVipGroupCharge);
					totalPrice=totalPrice.multiply(discount).divide(new BigDecimal(100)).setScale(2,BigDecimal.ROUND_HALF_UP);
					break;
				case GIVE_CLASS:
					if(totalClassNum.compareTo(new BigDecimal(vipGroupActivity.getAttribute1()))<0){

					}else if(vipGroup.getGiveTeachMode()==TeachModeEnum.OFFLINE){
						if(totalClassNum.compareTo(new BigDecimal(vipGroupActivity.getAttribute1()))>-1){
							offlineClassNum=offlineClassNum.subtract(new BigDecimal(vipGroupActivity.getAttribute2()));
						}
						offlineVipGroupCharge = offlineClassesUnitPrice.multiply(offlineClassNum);
					}else if(vipGroup.getGiveTeachMode()==TeachModeEnum.ONLINE){
						if(totalClassNum.compareTo(new BigDecimal(vipGroupActivity.getAttribute1()))>-1){
							onlineClassNum=onlineClassNum.subtract(new BigDecimal(vipGroupActivity.getAttribute2()));
						}
						onlineVipGroupCharge = onlineClassesUnitPrice.multiply(onlineClassNum);
					}else{
						throw new BizException("请指定赠送课程类型!");
					}
					totalPrice=onlineVipGroupCharge.add(offlineVipGroupCharge);
					break;
				default:
					throw new BizException("活动类型错误!");
			}
			results.put("totalPrice",totalPrice);
		}
        return results;
    }

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateVipGroupStudentNumAndStatus(Long vipGroupId , ClassGroup classGroup, Integer num, boolean updateVipStatus) {
		VipGroup vipGroup = vipGroupDao.getLockVipGroup(vipGroupId);
		if(Objects.isNull(vipGroup)){
			throw new BizException("指定的vip课程不存在");
		}
		if(Objects.isNull(classGroup)){
			throw new BizException("未找到关联班级");
		}
		if(classGroup.getStudentNum()+num<0){
			classGroup.setStudentNum(0);
		}else{
			classGroup.setStudentNum(classGroup.getStudentNum()+num);
		}

		Integer studentPaymentNum=studentPaymentOrderDao.countStudentPaymentNum(vipGroupId.toString());

		if(studentPaymentNum.equals(classGroup.getExpectStudentNum())&&updateVipStatus){
			classGroup.setStudentNum(studentPaymentNum);
			classGroup.setDelFlag(YesOrNoEnum.NO);
			vipGroup.setStatus(VipGroupStatusEnum.PROGRESS);
			vipGroupDao.update(vipGroup);
			createVipGroupCourseScheInfo(vipGroupId,classGroup);
		}
		if(num!=0||updateVipStatus){
			classGroupDao.update(classGroup);
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void createVipGroupCourseScheInfo(Long vipGroupId,ClassGroup classGroup){
		VipGroup vipGroupApplyBaseInfoDto = vipGroupDao.get(vipGroupId);
		List<CourseSchedule> courseSchedules = JSON.parseArray(vipGroupApplyBaseInfoDto.getCourseSchedulesJson(),CourseSchedule.class);
		courseScheduleService.batchAddCourseSchedule(courseSchedules);

		SysUser sysUser = sysUserFeignService.queryUserById(vipGroupApplyBaseInfoDto.getUserId());

		ClassGroupTeacherSalary classGroupTeacherSalary = classGroupTeacherSalaryDao.findByVipGoupAndTeacher(vipGroupId.intValue(), vipGroupApplyBaseInfoDto.getUserId());

		//创建老师单节课课酬信息
		courseScheduleTeacherSalaryService.createCourseScheduleTeacherVipSalary(vipGroupApplyBaseInfoDto,
				courseSchedules,
				classGroupTeacherSalary.getOnlineClassesSalary(),
				classGroupTeacherSalary.getSalary());

		List<ClassGroupStudentMapper> classGroupStudents = classGroupStudentMapperDao.findByClassGroup(classGroup.getId());

		List<ImGroupMember> imGroupMemberList = new ArrayList<>();
		imGroupMemberList.add(new ImGroupMember(vipGroupApplyBaseInfoDto.getUserId().toString()));
		//生成学生单课缴费信息
		for (ClassGroupStudentMapper classGroupStudent : classGroupStudents) {
			courseScheduleStudentPaymentService.createCourseScheduleStudentPaymentForVipGroup(vipGroupId,classGroupStudent.getUserId());
			imGroupMemberList.add(new ImGroupMember(classGroupStudent.getUserId().toString()));
		}

		try {
			ImGroupMember[] imGroupMembers = imGroupMemberList.toArray(new ImGroupMember[imGroupMemberList.size()]);
			// 创建群组
			imFeignService.groupCreate(new ImGroupModel(classGroup.getId().toString(), imGroupMembers, classGroup.getName()));

			//发送推送短信
			Map<Integer,String> map = new HashMap<>(1);
			map.put(vipGroupApplyBaseInfoDto.getUserId(),sysUser.getPhone());
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI,MessageTypeEnum.TEACHER_SMS_VIP_COURSE_ADD,
					map,null,0,"",vipGroupApplyBaseInfoDto.getName());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public Map buyVipGroup(VipGroupBuyParamsDto vipGroupBuyParams) throws Exception {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(user == null){
			throw new BizException("用户信息获取失败");
		}

		ClassGroup classGroup= classGroupDao.lockByMusicGroupAndType(vipGroupBuyParams.getVipGroupId().toString(),ClassGroupTypeEnum.VIP.getCode());
		if(Objects.isNull(classGroup)){
			throw new BizException("课程信息错误");
		}

		VipGroup vipGroup = vipGroupDao.get(vipGroupBuyParams.getVipGroupId());

		if(vipGroup.getStatus()!=VipGroupStatusEnum.APPLYING){
			throw new BizException("该课程已结束报名!");
		}

		StudentPaymentOrder oldOrder = studentPaymentOrderDao.findNotFailedOrderByStudentVipGroup(vipGroupBuyParams.getVipGroupId(),
				user.getId());

		if(Objects.nonNull(oldOrder)&&oldOrder.getStatus().equals(DealStatusEnum.SUCCESS)){
			throw new BizException("您已购买过此课程");
		}else if(Objects.nonNull(oldOrder)&&oldOrder.getStatus().equals(DealStatusEnum.ING)){
			throw new BizException("您有处理中的订单，请等待处理结果");
		}else{
			if(classGroup.getStudentNum()>=classGroup.getExpectStudentNum()){
				throw new BizException("该课程人数已达上限");
			}
			//更新班级人数，如果订单支付失败则减少
			this.updateVipGroupStudentNumAndStatus(vipGroup.getId(),classGroup,1,false);

			ClassGroupStudentMapper classGroupStudentMapper=new ClassGroupStudentMapper(classGroup.getId(),user.getId().intValue());
			classGroupStudentMapper.setGroupType(GroupType.VIP);
			classGroupStudentMapper.setMusicGroupId(vipGroup.getId().toString());
			classGroupStudentMapperDao.insert(classGroupStudentMapper);
		}

		Date date=new Date();

		List<CourseSchedule> courseSchedules = JSON.parseArray(vipGroup.getCourseSchedulesJson(),CourseSchedule.class);
		courseScheduleService.checkNewCourseSchedules(courseSchedules,false);

		StudentPaymentOrder studentPaymentOrder=new StudentPaymentOrder();
		studentPaymentOrder.setUserId(user.getId());
        studentPaymentOrder.setGroupType(GroupType.VIP);
		String orderNo=StringUtils.join(new String[]{user.getId().toString(),String.valueOf(System.currentTimeMillis())});
		studentPaymentOrder.setOrderNo(orderNo);
		studentPaymentOrder.setStatus(DealStatusEnum.ING);
		studentPaymentOrder.setType(OrderTypeEnum.SMALL_CLASS_TO_BUY);
		studentPaymentOrder.setExpectAmount(vipGroup.getTotalPrice());
		studentPaymentOrder.setMusicGroupId(vipGroup.getId().toString());
		studentPaymentOrder.setActualAmount(studentPaymentOrder.getExpectAmount());
		studentPaymentOrder.setMusicGroupId(vipGroup.getId().toString());
		studentPaymentOrder.setClassGroupId(classGroup.getId());
		studentPaymentOrder.setVersion(0);
		studentPaymentOrderDao.insert(studentPaymentOrder);

		BigDecimal amount=vipGroup.getTotalPrice();

		if(vipGroupBuyParams.isUseBalancePayment() || amount.doubleValue() == 0){
			SysUserCashAccount userCashAccount = sysUserCashAccountService.getLocked(vipGroupBuyParams.getUserId());
			if(userCashAccount == null){
				throw new BizException("用户账户找不到");
			}
			if(userCashAccount.getBalance().subtract(vipGroup.getTotalPrice()).doubleValue() > 0){
				// 更新订单信息
				studentPaymentOrder.setBalancePaymentAmount(amount);
				studentPaymentOrder.setStatus(DealStatusEnum.SUCCESS);
				studentPaymentOrder.setUpdateTime(date);
				studentPaymentOrderService.update(studentPaymentOrder);
				studentPaymentOrder.setVersion(studentPaymentOrder.getVersion()+1);

				sysUserCashAccountService.updateBalance(vipGroupBuyParams.getUserId(), amount.negate(),PlatformCashAccountDetailTypeEnum.PAY_FEE,"乐团续费");

				return null;
			}else{
				if (userCashAccount.getBalance().doubleValue() > 0) {
					sysUserCashAccountService.updateBalance(vipGroupBuyParams.getUserId(), userCashAccount.getBalance().negate(), PlatformCashAccountDetailTypeEnum.PAY_FEE, "乐团续费");
					amount = amount.subtract(userCashAccount.getBalance());
					studentPaymentOrder.setBalancePaymentAmount(userCashAccount.getBalance());
				} else {
					studentPaymentOrder.setBalancePaymentAmount(new BigDecimal(0));
				}
			}
		}

//		try {
//			contractService.register(user.getId(),user.getRealName(),user.getIdCardNo(),user.getPhone());
//		} catch (Exception e) {
//			e.printStackTrace();
//		}

		try {
			Map<String,Object> payMap = payService.getPayMap(
					amount,
					orderNo,
					"http://mstudev.dayaedu.com/api-student/studentOrder/notify",
					"http://mstudev.dayaedu.com/#/paymentresult?orderNo=" + orderNo,
					"vip课购买",
					vipGroup.getName());
			studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
			studentPaymentOrder.setUpdateTime(date);
			studentPaymentOrderService.update(studentPaymentOrder);

			return payMap;
		} catch (Exception e) {
			throw new BizException("调用支付接口出错", e);
		}

	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void orderCallback(StudentPaymentOrder order) {
		/*
		根据回调信息调整订单状态及vip课程状态等相关信息
		 */
		Long vipGroupId=Long.parseLong(order.getMusicGroupId());
		Integer userId = order.getUserId();
		boolean isOk=order.getStatus().equals(DealStatusEnum.SUCCESS);

		StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.get(order.getId());
		if(studentPaymentOrder.getStatus().equals(DealStatusEnum.SUCCESS)){
			return;
		}

		studentPaymentOrderDao.update(order);

		ClassGroup classGroup=classGroupDao.lockByMusicGroupAndType(vipGroupId.toString(),ClassGroupTypeEnum.VIP.getCode());

		//将学生加入到班级，更新班级报名状态及人数信息
		if(!isOk){
			updateVipGroupStudentNumAndStatus(vipGroupId,classGroup,-1,false);
			classGroupStudentMapperDao.deleteStudentByMusicGroupId(vipGroupId.toString(),userId);
			return;
		}

		updateVipGroupStudentNumAndStatus(vipGroupId, classGroup,0,true);
		SysUserCashAccount sysUserCashAccount = sysUserCashAccountService.get(userId);

		VipGroup vipGroup = vipGroupDao.get(vipGroupId);
		//插入缴费明细
		//收入
		SysUserCashAccountDetail sysUserIncomeCashAccountDetail = new SysUserCashAccountDetail();
		sysUserIncomeCashAccountDetail.setUserId(userId);
		sysUserIncomeCashAccountDetail.setType(PlatformCashAccountDetailTypeEnum.RECHARGE);
		sysUserIncomeCashAccountDetail.setStatus(DealStatusEnum.SUCCESS);
		sysUserIncomeCashAccountDetail.setAmount(order.getActualAmount());
		sysUserIncomeCashAccountDetail.setBalance(sysUserCashAccount.getBalance().add(order.getActualAmount()));
		sysUserIncomeCashAccountDetail.setAttribute(order.getTransNo());

		//支出
		SysUserCashAccountDetail sysUserExpendCashAccountDetail = new SysUserCashAccountDetail();
		sysUserExpendCashAccountDetail.setUserId(userId);
		sysUserExpendCashAccountDetail.setType(PlatformCashAccountDetailTypeEnum.PAY_FEE);
		sysUserExpendCashAccountDetail.setStatus(DealStatusEnum.SUCCESS);
		sysUserExpendCashAccountDetail.setAmount(order.getActualAmount().negate());
		sysUserExpendCashAccountDetail.setBalance(sysUserCashAccount.getBalance());
		sysUserExpendCashAccountDetail.setAttribute(order.getTransNo());

		sysUserCashAccountDetailService.insert(sysUserIncomeCashAccountDetail);
		sysUserCashAccountDetailService.insert(sysUserExpendCashAccountDetail);
		Map<Integer,String> map = new HashMap<>(1);
		map.put(userId,userId.toString());
		sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_VIP_BUY, map, null, 0, "2",
				vipGroup.getName());

		//更新所属分部列表
		List<Integer> organIds = classGroupDao.findStudentOrganIdsByClassGroup(classGroup.getId().longValue());
		organIds.add(vipGroup.getOrganId());
		HashSet<Integer> hashSet = new HashSet<>(organIds);
		String organIdsString = StringUtils.join(hashSet, ",");
		vipGroup.setOrganIdList(organIdsString);
		vipGroupDao.update(vipGroup);

//		try {
//			contractService.transferVipGroupCoursesContract(userId,vipGroupId);
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void applyRefund(Long vipGroupId,Integer studentId) {

		StudentApplyRefunds studentApplyRefunds=new StudentApplyRefunds();
		StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.findByStudentVipGroup(vipGroupId,studentId,DealStatusEnum.SUCCESS.getCode());
		if(null==studentPaymentOrder){
			throw new BizException("未找到相关订单信息！");
		}
		VipGroup vipGroup=vipGroupDao.get(vipGroupId);
		//剩余课时数
		Map surplusClassTimes=MapUtil.convertMybatisMap(vipGroupDao.countSurplusClassTimes(vipGroupId));
		studentApplyRefunds.setExpectAmount(countVipGroupPredictFee(vipGroup,
				vipGroup.getUserId(),
				new BigDecimal(Objects.isNull(surplusClassTimes.get(TeachModeEnum.ONLINE.getCode()))?"0":surplusClassTimes.get(TeachModeEnum.ONLINE.getCode()).toString()),
				new BigDecimal(Objects.isNull(surplusClassTimes.get(TeachModeEnum.OFFLINE.getCode()))?"0":surplusClassTimes.get(TeachModeEnum.OFFLINE.getCode()).toString()),
				true,
				false).get("totalPrice"));
		studentApplyRefunds.setStatus(StudentApplyRefundsStatus.ING);
		String orderNo=StringUtils.join(new String[]{studentId.toString(),String.valueOf(System.currentTimeMillis())});
		studentApplyRefunds.setOrderNo(orderNo);
		studentApplyRefunds.setOrigPaymentOrderId(studentPaymentOrder.getId());
		studentApplyRefunds.setUserId(studentId.intValue());
		studentApplyRefundsDao.insert(studentApplyRefunds);
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		Set<Integer> roleIds = new HashSet<>(1);
		roleIds.add(SysUserRole.EDUCATIONAL_TEACHER);
		Map<String,Object> memo = new HashMap<>(2);
		memo.put("Id",vipGroupId);
		memo.put("type","VIPGROUP");
		sysMessageService.batchSeoMessage(musicGroupDao.queryUserIdByRoleId(roleIds),MessageTypeEnum.BACKSTAGE_STUDENT_APPLY_QUIT_COURSE,JSONObject.toJSONString(memo),sysUser.getUsername());
	}

	@Override
	public void applyRefundForStudent(Long vipGroupId, Integer studentId, BigDecimal amount) {
		if(Objects.isNull(vipGroupId)||Objects.isNull(studentId)){
			throw new BizException("请指定小课与学生");
		}
		if(Objects.isNull(amount)){
			throw new BizException("请确定退费金额");
		}
		VipGroup vipGroup=vipGroupDao.get(vipGroupId);
		if(Objects.isNull(vipGroup)){
			throw new BizException("指定的课程不存在");
		}
		if(vipGroup.getStatus().equals(VipGroupStatusEnum.CANCEL)){
			throw new BizException("不能已进行的课程进行此操作");
		}
		ClassGroup classGroup = classGroupDao.findByVipGroup(vipGroupId, null);
		sysUserCashAccountService.updateBalance(studentId, amount);
		SysUserCashAccount sysUserCashAccount = sysUserCashAccountService.get(studentId);
		SysUserCashAccountDetail sysUserCashAccountDetail = new SysUserCashAccountDetail();
		sysUserCashAccountDetail.setUserId(studentId);
		sysUserCashAccountDetail.setType(PlatformCashAccountDetailTypeEnum.REFUNDS);
		sysUserCashAccountDetail.setStatus(DealStatusEnum.SUCCESS);
		sysUserCashAccountDetail.setAmount(amount);
		sysUserCashAccountDetail.setBalance(sysUserCashAccount.getBalance());
		sysUserCashAccountDetail.setAttribute(studentId.toString());
		sysUserCashAccountDetailDao.insert(sysUserCashAccountDetail);

		ClassGroupStudentMapper classStudentMapperByUserIdAndClassGroupId = classGroupStudentMapperDao.query(classGroup.getId(),
				studentId);

		if(Objects.isNull(classStudentMapperByUserIdAndClassGroupId)){
			throw new BizException("指定学生不在此课程中");
		}

		classStudentMapperByUserIdAndClassGroupId.setStatus(ClassGroupStudentStatusEnum.QUIT);
		classGroupStudentMapperDao.update(classStudentMapperByUserIdAndClassGroupId);

		courseScheduleService.batchDeleteMusicGroupCourseWithStudent(vipGroupId.toString(),studentId,GroupType.VIP);

		classGroup.setStudentNum(classGroupStudentMapperDao.countClassGroupStudentNum(classGroup.getId()));
		classGroupDao.update(classGroup);

		//学员退出班级群
		ImGroupMember[] imGroupMembers = new ImGroupMember[]{new ImGroupMember(studentId.toString())};
		imFeignService.groupJoin(new ImGroupModel(classGroup.getId().toString(), imGroupMembers, null));
	}

	@Override
	public Map<String, BigDecimal> getStudentSurplusCourseFee(Long vipGroupId, Integer studentId) {
		if(Objects.isNull(vipGroupId)||Objects.isNull(studentId)){
			throw new BizException("请指定课程和学生");
		}
		VipGroup vipGroup = vipGroupDao.get(vipGroupId);
		if(Objects.isNull(vipGroup)){
			throw new BizException("未找到指定vip课");
		}
		if(vipGroup.getStatus().equals(VipGroupStatusEnum.CANCEL)){
			throw new BizException("不能对已停止的课程进行退课操作");
		}
		Map<String,BigDecimal> result=new HashMap<>();
		if(vipGroup.getStatus().equals(VipGroupStatusEnum.APPLYING)){
			StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.findByStudentVipGroup(vipGroupId, studentId, DealStatusEnum.SUCCESS.getCode());
			result.put("suplusCourseFee",studentPaymentOrder.getActualAmount());
			return result;
		}
		ClassGroup classGroup = classGroupDao.findByVipGroup(vipGroupId, null);
		if(Objects.isNull(classGroup)){
			throw new BizException("为找到对应班级");
		}
		BigDecimal bigDecimal = courseScheduleStudentPaymentDao.countSurplusCourseFee(classGroup.getId(), studentId);
		result.put("suplusCourseFee",Objects.isNull(bigDecimal)?new BigDecimal(0):bigDecimal);
		return result;
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void applyRefundAudit(Long id, StudentApplyRefundsStatus status, String remark,BigDecimal amount) {

		StudentApplyRefunds studentApplyRefunds = studentApplyRefundsDao.get(id);
		if (studentApplyRefunds == null) {
			throw new BizException("退课申请记录不存在");
		}

		StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.get(studentApplyRefunds.getOrigPaymentOrderId());
		if (null == studentPaymentOrder) {
			throw new BizException("未找到原订单信息！");
		}
		
		if(status ==StudentApplyRefundsStatus.ING){
			throw new BizException("状态不正确");
		}

		studentApplyRefunds.setStatus(status);
		if (studentApplyRefunds.getStatus() == StudentApplyRefundsStatus.DONE) {
			studentApplyRefunds.setActualAmount(amount);
		} else if (studentApplyRefunds.getStatus() == StudentApplyRefundsStatus.WAIT_PAYMENT){
			studentApplyRefunds.setExpectAmount(amount);
		}else{
			studentApplyRefunds.setActualAmount(new BigDecimal(0));
		}
		studentApplyRefunds.setRemark(remark);
		studentApplyRefunds.setUpdateTime(new Date());
		studentApplyRefundsDao.update(studentApplyRefunds);
		switch (studentApplyRefunds.getStatus()) {
			case DONE:
				sysUserCashAccountService.updateBalance(studentApplyRefunds.getUserId(), studentApplyRefunds.getActualAmount());
				SysUserCashAccount sysUserCashAccount = sysUserCashAccountService.get(studentApplyRefunds.getUserId().intValue());
				SysUserCashAccountDetail sysUserCashAccountDetail = new SysUserCashAccountDetail();
				sysUserCashAccountDetail.setUserId(studentApplyRefunds.getUserId());
				sysUserCashAccountDetail.setType(PlatformCashAccountDetailTypeEnum.REFUNDS);
				sysUserCashAccountDetail.setStatus(DealStatusEnum.SUCCESS);
				sysUserCashAccountDetail.setAmount(amount);
				sysUserCashAccountDetail.setBalance(sysUserCashAccount.getBalance());
				sysUserCashAccountDetail.setAttribute(studentApplyRefunds.getId().toString());
				sysUserCashAccountDetailDao.insert(sysUserCashAccountDetail);

				ClassGroupStudentMapper classStudentMapperByUserIdAndClassGroupId = classGroupStudentMapperDao.query(studentPaymentOrder.getClassGroupId(),
						studentPaymentOrder.getUserId());

				classStudentMapperByUserIdAndClassGroupId.setStatus(ClassGroupStudentStatusEnum.QUIT);
				classGroupStudentMapperDao.update(classStudentMapperByUserIdAndClassGroupId);

				ClassGroup classGroup=new ClassGroup();
				classGroup.setId(classStudentMapperByUserIdAndClassGroupId.getClassGroupId());
				classGroup.setStudentNum(classGroupStudentMapperDao.countClassGroupStudentNum(classGroup.getId()));
				classGroupDao.update(classGroup);

				courseScheduleService.batchDeleteMusicGroupCourseWithStudent(studentPaymentOrder.getMusicGroupId(),studentApplyRefunds.getUserId(),GroupType.VIP);

				//学员退出班级群
				ImGroupMember[] imGroupMembers = new ImGroupMember[]{new ImGroupMember(studentApplyRefunds.getUserId().toString())};
				imFeignService.groupJoin(new ImGroupModel(studentPaymentOrder.getClassGroupId().toString(), imGroupMembers, null));
				break;
			case REJECT:

				break;
			case WAIT_PAYMENT:

				break;
			default:
				throw new BizException("审核状态错误!");
		}
	}

	@Override
	public Map<String, Object> findVipGroupSalarys(VipGroupSalaryQueryInfo queryInfo) {

    	if(Objects.isNull(queryInfo.getVipGroupId())){
    		throw new BizException("请指定vip课程!");
		}

		VipGroup vipGroup = vipGroupDao.get(queryInfo.getVipGroupId());

    	if(Objects.isNull(vipGroup)){
    		throw new BizException("未找到指定vip课");
		}

		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<VipGroupSalaryDto> dataList = null;
		int count = courseScheduleTeacherSalaryDao.countVipGroupSalarys(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = courseScheduleTeacherSalaryDao.findVipGroupSalarys(params);
			if(!CollectionUtils.isEmpty(dataList)){
				dataList.forEach(vipGroupSalaryDto -> {
					if(Objects.isNull(vipGroupSalaryDto.getExpectSalary())){
						vipGroupSalaryDto.setExpectSalary(new BigDecimal(0));
					}
					if(Objects.isNull(vipGroupSalaryDto.getActualSalary())){
						vipGroupSalaryDto.setActualSalary(new BigDecimal(0));
						vipGroupSalaryDto.setDeductionFee(new BigDecimal(0));
						vipGroupSalaryDto.setIsSalary(YesOrNoEnum.NO);
					}else{
						vipGroupSalaryDto.setDeductionFee(vipGroupSalaryDto.getExpectSalary().subtract(vipGroupSalaryDto.getActualSalary()));
						vipGroupSalaryDto.setIsSalary(YesOrNoEnum.YES);
					}
				});
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);

		VipGroupSalaryBaseInfo vipGroupSalaryBaseInfo = vipGroupDao.getVipGroupSalaryInfo(queryInfo.getVipGroupId());
		if(Objects.isNull(vipGroupSalaryBaseInfo)){
			vipGroupSalaryBaseInfo=new VipGroupSalaryBaseInfo();
		}
		//预计总课酬
		if(Objects.isNull(vipGroupSalaryBaseInfo.getExpectTotalSalary())){
			vipGroupSalaryBaseInfo.setExpectTotalSalary(new BigDecimal(0));
		}
		//实际总课酬
		if(Objects.isNull(vipGroupSalaryBaseInfo.getTotalSalary())){
			vipGroupSalaryBaseInfo.setTotalSalary(new BigDecimal(0));
		}

		ClassGroup classGroup = classGroupDao.findByVipGroup(vipGroup.getId(),null);

		//扣费笔数
		vipGroupSalaryBaseInfo.setFeeDeductionNum(vipGroupDao.countStudentPaymentNum(classGroup.getId()));
		//扣费总额
		if(vipGroupSalaryBaseInfo.getFeeDeductionNum()>0){
			vipGroupSalaryBaseInfo.setTotalFeeDeduction(vipGroupDao.countStudentPaymentFee(classGroup.getId()));
		}else{
			vipGroupSalaryBaseInfo.setTotalFeeDeduction(new BigDecimal(0));
		}

		BigDecimal totalCoursePrice= courseScheduleStudentPaymentDao.countAllCourseFee(classGroup.getId());
		if(Objects.isNull(totalCoursePrice)){
			totalCoursePrice=new BigDecimal(0);
		}
		//课程总价
		vipGroupSalaryBaseInfo.setTotalCoursePrice(totalCoursePrice);

		List<Map<String, BigDecimal>> teacherSalaryCount = courseScheduleTeacherSalaryDao.countTeacherOnlineOfflineSalaryByClass(classGroup.getId());
		Map<String,BigDecimal> teacherTeachModeSalaryMap = MapUtil.convertIntegerMap(teacherSalaryCount);
		vipGroupSalaryBaseInfo.setTotalOfflineTeacherSalary(teacherTeachModeSalaryMap.get(TeachModeEnum.OFFLINE.getCode()));
		vipGroupSalaryBaseInfo.setTotalOnlineTeacherSalary(teacherTeachModeSalaryMap.get(TeachModeEnum.ONLINE.getCode()));

		Map<String,Object> result=new HashMap<>();
		result.put("pageInfo",pageInfo);
		result.put("baseInfo",vipGroupSalaryBaseInfo);
		return result;
	}

	@Override
	public List<String> findVipGroupAttendanceStudents(Long courseScheduleId) {
    	if(Objects.isNull(courseScheduleId)){
    		throw new BizException("请指定课程");
		}
		return courseScheduleDao.findStudentNamesByCourseSchedule(courseScheduleId);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void stopGroup(Long vipGroupId,String stopReason) {
    	if(Objects.isNull(vipGroupId)){
    		throw new BizException("请指定vip课");
		}
		VipGroup vipGroup = vipGroupDao.get(vipGroupId);
    	if(Objects.isNull(vipGroup)){
    		throw new BizException("该课程不存在");
		}
    	if(vipGroup.getStatus()==VipGroupStatusEnum.CANCEL){
    		throw new BizException("该课程已结束或者已被停止,无法进行此操作");
		}
		List<CourseSchedule> vipGroupCourseSchedules = courseScheduleDao.findVipGroupCourseSchedules(vipGroupId);
		List<Long> courseScheduleIds = vipGroupCourseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toList());
    	if(vipGroup.getStatus()==VipGroupStatusEnum.NOT_START){
			vipGroup.setStatus(VipGroupStatusEnum.CANCEL);
    		vipGroupDao.update(vipGroup);
			courseScheduleDao.batchDeleteCourseSchedules(courseScheduleIds);
			courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
    		return;
		}
		List<Map<Integer, BigDecimal>> maps = vipGroupDao.countSurplusCourseFee(vipGroupId);
    	if(CollectionUtils.isEmpty(maps)||(maps.size()==1&&Objects.isNull(maps.get(0)))){
			vipGroup.setStatus(VipGroupStatusEnum.CANCEL);
			vipGroupDao.update(vipGroup);
			if(vipGroup.getStatus().equals(VipGroupStatusEnum.PROGRESS)){
				courseScheduleDao.batchDeleteCourseSchedules(courseScheduleIds);
				courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
			}
			return;
		}
		Map<Integer, BigDecimal> studentSurplusClassFees = MapUtil.convertIntegerMap(maps);
    	for(Integer userId:studentSurplusClassFees.keySet()){
			StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.findByStudentVipGroup(vipGroupId,userId,DealStatusEnum.SUCCESS.getCode());
			if(null==studentPaymentOrder){
				throw new BizException("未找到相关订单信息！");
			}
			sysUserCashAccountService.updateBalance(userId.intValue(),studentSurplusClassFees.get(userId));
			SysUserCashAccount sysUserCashAccount = sysUserCashAccountService.get(userId.intValue());
			SysUserCashAccountDetail sysUserCashAccountDetail=new SysUserCashAccountDetail();
			sysUserCashAccountDetail.setUserId(userId.intValue());
			sysUserCashAccountDetail.setType(PlatformCashAccountDetailTypeEnum.REFUNDS);
			sysUserCashAccountDetail.setStatus(DealStatusEnum.SUCCESS);
			sysUserCashAccountDetail.setAmount(studentSurplusClassFees.get(userId));
			sysUserCashAccountDetail.setBalance(sysUserCashAccount.getBalance());
			sysUserCashAccountDetail.setAttribute(studentPaymentOrder.getOrderNo());
			sysUserCashAccountDetailDao.insert(sysUserCashAccountDetail);
		}

		vipGroup.setStatus(VipGroupStatusEnum.CANCEL);
    	vipGroup.setStopReason(stopReason);
		vipGroupDao.update(vipGroup);

		courseScheduleDao.batchDeleteCourseSchedules(courseScheduleIds);
		courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
		courseScheduleStudentPaymentDao.deleteByCourseSchedule(courseScheduleIds);
		teacherAttendanceDao.batchDeleteByCourseSchedules(courseScheduleIds);
		//vip课停止通知
		Teacher teacher = teacherDao.get(vipGroup.getUserId());
		Map<Integer,String> map = new HashMap<>(1);
		map.put(vipGroup.getUserId(),vipGroup.getUserId().toString());
		sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,MessageTypeEnum.TEACHER_PUSH_VIP_COURSE_STOP,
				map,null,0,"1",vipGroup.getName());
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
	public boolean awardedMonthlyRewards() {
		List<Map<Integer, Integer>> list = courseScheduleDao.queryVipGroupTeachereClassTimesByMonth(new Date(), CourseStatusEnum.OVER);

		if (list != null && list.size() > 0) {

			Integer organId = null, teacherId = null, coursesTimes = 0;
			boolean isSupportCourseScheduleRewardsRules = false;

			for (Map<Integer, Integer> map : list) {
				organId = map.get("organ_id_");
				teacherId = map.get("teacher_id_");
				coursesTimes = map.get("times");
				isSupportCourseScheduleRewardsRules = Boolean.parseBoolean(map.get("is_support_course_schedule_rewards_rules_").toString());

				if (isSupportCourseScheduleRewardsRules) {
					BigDecimal decimal = courseScheduleRewardsRulesService.queryRewardsAmount(organId, CourseScheduleType.VIP, coursesTimes);
					if (decimal.doubleValue() > 0) {
						sysUserCashAccountDetailService.addCashAccountDetail(teacherId, decimal, "", SysUserCashAccountDetailService.ORGAN,
								PlatformCashAccountDetailTypeEnum.REWARDS, null, DealStatusEnum.SUCCESS, "分部奖励");
					}
				}
			}
		}

		return true;
	}

	@Override
	public List<School> querySchoolByUserId(Integer userId) {
		return vipGroupDao.querySchoolByUserId(userId);
	}

	@Override
	public void updateVipGroupBaseInfo(VipGroupApplyBaseInfoDto vipGroupApplyBaseInfo) {
		if(Objects.isNull(vipGroupApplyBaseInfo.getId())){
			throw new BizException("请指定vip课");
		}
		VipGroup oldVipGroupInfo = vipGroupDao.get(vipGroupApplyBaseInfo.getId());
		if(Objects.isNull(oldVipGroupInfo)){
			throw new BizException("指定的vip课不存在");
		}
		Date now=new Date();
		//更新vip课
		vipGroupApplyBaseInfo.setUpdateTime(now);
		vipGroupDao.update(vipGroupApplyBaseInfo);

	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void appendVipGroupCourseSchedules(VipGroupApplyDto vipGroupApplyDto) {
		if(Objects.isNull(vipGroupApplyDto.getVipGroupApplyBaseInfo().getId())){
			throw new BizException("请指定vip课");
		}
		VipGroup vipGroup = vipGroupDao.get(vipGroupApplyDto.getVipGroupApplyBaseInfo().getId());
		if(Objects.isNull(vipGroup)){
			throw new BizException("指定的vip课不存在");
		}
		ClassGroup classGroup = classGroupDao.findByVipGroup(vipGroup.getId(), null);
		//课程信息调整
		vipGroupApplyDto.getCourseSchedules().forEach(courseSchedule -> {
			courseSchedule.setSchoolId(vipGroup.getTeacherSchoolId());
			courseSchedule.setTeacherId(vipGroup.getUserId());
			courseSchedule.setActualTeacherId(vipGroup.getUserId());
			courseSchedule.setStatus(CourseStatusEnum.NOT_START);
			courseSchedule.setType(CourseSchedule.CourseScheduleType.VIP);
			courseSchedule.setClassGroupId(classGroup.getId());
			courseSchedule.setGroupType(GroupType.MUSIC);
			courseSchedule.setMusicGroupId(vipGroup.getId().toString());
		});
		courseScheduleService.batchAddCourseSchedule(vipGroupApplyDto.getCourseSchedules());

		TeacherDefaultVipGroupSalary byTeacherAndCategory = teacherDefaultVipGroupSalaryDao.findByTeacherAndCategory(vipGroup.getUserId(), vipGroup.getVipGroupCategoryId());

		BigDecimal onlineTeacherSalary=new BigDecimal(0),offlineTeacherSalary=new BigDecimal(0);
		if(Objects.nonNull(byTeacherAndCategory)){
			onlineTeacherSalary=byTeacherAndCategory.getOnlineClassesSalary();
			offlineTeacherSalary=byTeacherAndCategory.getOfflineClassesSalary();
		}else{
			ClassGroupTeacherSalary classGroupTeacherSalary = classGroupTeacherSalaryDao.findByVipGoupAndTeacher(vipGroup.getId().intValue(), vipGroup.getUserId());
			onlineTeacherSalary=classGroupTeacherSalary.getOnlineClassesSalary();
			offlineTeacherSalary=classGroupTeacherSalary.getSalary();
		}

		//创建老师单节课课酬信息
		courseScheduleTeacherSalaryService.createCourseScheduleTeacherVipSalary(vipGroup,
				vipGroupApplyDto.getCourseSchedules(),
				onlineTeacherSalary,
				offlineTeacherSalary);


		List<ClassGroupStudentMapper> classGroupStudents = classGroupStudentMapperDao.findByClassGroup(classGroup.getId());

		List<ImGroupMember> imGroupMemberList = new ArrayList<>();
		//生成学生单课缴费信息
		for (ClassGroupStudentMapper classGroupStudent : classGroupStudents) {
			courseScheduleStudentPaymentService.createCourseScheduleStudentPaymentForVipGroup(vipGroup.getId(),classGroupStudent.getUserId());
			imGroupMemberList.add(new ImGroupMember(classGroupStudent.getUserId().toString()));
		}
		ImGroupMember[] imGroupMembers = imGroupMemberList.toArray(new ImGroupMember[imGroupMemberList.size()]);
		imFeignService.groupJoin(new ImGroupModel(classGroup.getId().toString(),
				imGroupMembers, classGroup.getName()));
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void addVipGroupStudents(Long vipGroupId, List<Integer> studentIds) {
		if(CollectionUtils.isEmpty(studentIds)){
			throw new BizException("请选择学生");
		}
		if(Objects.isNull(vipGroupId)){
			throw new BizException("请指定vip课程");
		}
		VipGroup vipGroup=vipGroupDao.get(vipGroupId);
		if(Objects.isNull(vipGroup)){
			throw new BizException("未找到指定的vip课程");
		}
		ClassGroup classGroup = classGroupDao.findByVipGroup(vipGroupId, null);

		Integer exitStudentNum = classGroupStudentMapperDao.countClassGroupExitStudentNum(classGroup.getId(), studentIds);
		if(exitStudentNum>0){
			throw new BizException("选择的学生中存在此课程中已存在的学生");
		}

		Date now=new Date();

		List<CourseSchedule> surplusCourses = courseScheduleDao.findByClassGroupAndStatus(classGroup.getId(), CourseStatusEnum.NOT_START.getCode());
		if(CollectionUtils.isEmpty(surplusCourses)){
			throw new BizException("此vip课程没有剩余课程计划");
		}

		BigDecimal surplusCoursesPrice=new BigDecimal(0);
		for (CourseSchedule courseSchedule : surplusCourses) {
			if(courseSchedule.getTeachMode().equals(TeachModeEnum.OFFLINE)){
				surplusCoursesPrice=surplusCoursesPrice.add(vipGroup.getOfflineClassesUnitPrice());
			}else if(courseSchedule.getTeachMode().equals(TeachModeEnum.ONLINE)){
				surplusCoursesPrice=surplusCoursesPrice.add(vipGroup.getOnlineClassesUnitPrice());
			}
		}
		List<CourseSchedule> vipGroupCourseSchedules=courseScheduleDao.findVipGroupCourseSchedules(vipGroupId);

		if(CollectionUtils.isEmpty(vipGroupCourseSchedules)){
			throw new BizException("此vip课不存在课程");
		}

		List<ImGroupMember> imGroupMemberList = new ArrayList<>();
		List<ClassGroupStudentMapper> classGroupStudentMappers=new ArrayList<>();
		//生成学生单课缴费信息
		for (Integer studentId:studentIds) {
			SysUserCashAccount sysUserCashAccount = sysUserCashAccountService.get(studentId);
			if(sysUserCashAccount.getCourseBalance().compareTo(surplusCoursesPrice)<0){
				throw new BizException("存在课程余额不足的学生");
			}
//			courseScheduleStudentPaymentService.createCourseScheduleStudentPaymentForVipGroup(vipGroupId,studentId);

			List<CourseScheduleStudentPayment> courseScheduleStudentPayments=new ArrayList<>();
			for (CourseSchedule vipGroupCourseSchedule : vipGroupCourseSchedules) {
				CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
				courseScheduleStudentPayment.setGroupType(GroupType.VIP);
				courseScheduleStudentPayment.setMusicGroupId(vipGroupId.toString());
				courseScheduleStudentPayment.setClassGroupId(vipGroupCourseSchedule.getClassGroupId());
				courseScheduleStudentPayment.setCourseScheduleId(vipGroupCourseSchedule.getId());
				courseScheduleStudentPayment.setUserId(studentId);
				courseScheduleStudentPayment.setExpectPrice(new BigDecimal(0));
				courseScheduleStudentPayment.setCreateTime(now);
				courseScheduleStudentPayment.setUpdateTime(now);
				courseScheduleStudentPayments.add(courseScheduleStudentPayment);
			}
			courseScheduleStudentPaymentDao.batchInsert(courseScheduleStudentPayments);

			sysUserCashAccountService.updateCourseBalance(studentId,sysUserCashAccount.getCourseBalance().subtract(surplusCoursesPrice));

			//创建班级学生关联记录
			ClassGroupStudentMapper classGroupStudentMapper=new ClassGroupStudentMapper();
			classGroupStudentMapper.setGroupType(classGroup.getGroupType());
			classGroupStudentMapper.setMusicGroupId(classGroup.getMusicGroupId());
			classGroupStudentMapper.setClassGroupId(classGroup.getId());
			classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);
			classGroupStudentMapper.setUserId(studentId);
			classGroupStudentMappers.add(classGroupStudentMapper);

			imGroupMemberList.add(new ImGroupMember(studentId.toString()));
		}

		classGroupStudentMapperDao.classGroupStudentsInsert(classGroupStudentMappers);

		classGroup.setStudentNum(classGroupStudentMapperDao.countClassGroupStudentNum(classGroup.getId()));
		classGroupDao.update(classGroup);

		try {
			ImGroupMember[] imGroupMembers = imGroupMemberList.toArray(new ImGroupMember[imGroupMemberList.size()]);
			// 创建群组
			imFeignService.groupCreate(new ImGroupModel(classGroup.getId().toString(), imGroupMembers, classGroup.getName()));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void stopVipGroupWithNoCreate() {
		List<VipGroup> noCreateSuccessVipGroups = vipGroupDao.findNoCreateSuccessVipGroup();
		if(CollectionUtils.isEmpty(noCreateSuccessVipGroups)){
			return;
		}
		//用户账户资金变动信息列表
		List<SysUserCashAccountDetail> userCashAccountDetails = new ArrayList<>();
		for (VipGroup noCreateSuccessVipGroup : noCreateSuccessVipGroups) {
			List<StudentPaymentOrder> studentPaymentOrders = studentPaymentOrderDao.queryByDealStatus(noCreateSuccessVipGroup.getId().toString(),
					OrderTypeEnum.SMALL_CLASS_TO_BUY, DealStatusEnum.SUCCESS);
			for (StudentPaymentOrder studentPaymentOrder:studentPaymentOrders){
				//生成账户资金明细
				sysUserCashAccountService.updateCourseBalance(studentPaymentOrder.getUserId(),studentPaymentOrder.getActualAmount());
				SysUserCashAccount sysUserCashAccount = sysUserCashAccountService.get(studentPaymentOrder.getUserId());
				SysUserCashAccountDetail sysUserIncomeCashAccountDetail = new SysUserCashAccountDetail();
				sysUserIncomeCashAccountDetail.setUserId(studentPaymentOrder.getUserId());
				sysUserIncomeCashAccountDetail.setType(PlatformCashAccountDetailTypeEnum.REFUNDS);
				sysUserIncomeCashAccountDetail.setStatus(DealStatusEnum.SUCCESS);
				sysUserIncomeCashAccountDetail.setAmount(studentPaymentOrder.getActualAmount());
				sysUserIncomeCashAccountDetail.setBalance(sysUserCashAccount.getBalance());
				sysUserIncomeCashAccountDetail.setAttribute(studentPaymentOrder.getTransNo());
				userCashAccountDetails.add(sysUserIncomeCashAccountDetail);
			}
		}
		sysUserCashAccountDetailDao.batchInsert(userCashAccountDetails);
	}

	@Override
	public void updateVipGroupStatusToFinished() {
		
	}
}
