package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dto.TeacherSalaryDto;
import com.ym.mec.biz.dal.entity.CourseScheduleTeacherSalary;
import com.ym.mec.biz.dal.page.CourseScheduleTeacherSalaryQueryInfo;
import com.ym.mec.biz.service.CourseScheduleTeacherSalaryService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.excel.POIUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RequestMapping("courseScheduleTeacherSalary")
@Api(tags = "教师薪酬服务")
@RestController
public class CourseScheduleTeacherSalaryController extends BaseController {

    @Autowired
    private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;

    @ApiOperation(value = "分页导出教师薪酬列表")
    @GetMapping("/export")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleTeacherSalary/export')")
    public void export(CourseScheduleTeacherSalaryQueryInfo queryInfo, HttpServletResponse response) {
        List<TeacherSalaryDto> teacherSalaries = courseScheduleTeacherSalaryService.querySalaries(queryInfo).getRows();
        OutputStream ouputStream = null;
        try {
            HSSFWorkbook workbook = POIUtil.exportExcel(new String[] { "用户编号","用户名", "手机号", "课程类型", "实际薪水", "补助", "结算时间", "教师角色"}, new String[] {
                    "userId","username", "phone", "type.msg", "name", "actualSalary", "subsidy", "settlementTime", "teacherRole.msg"}, teacherSalaries);
            response.setContentType("application/msexcel");
            response.setHeader("Content-disposition", "attachment;filename=lender-" + DateUtil.getDate(new Date()) + ".xls");
            ouputStream = response.getOutputStream();
            workbook.write(ouputStream);
            ouputStream.flush();
        } catch (Exception e) {
            e.printStackTrace();
//            LOGGER.error("导出借款人列表出现异常", e);
        } finally {
            if (ouputStream != null) {
                try {
                    ouputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
//                    LOGGER.error("输出流关闭异常", e);
                }
            }
        }
    }

    @ApiOperation(value = "分页查询教师薪酬列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleTeacherSalary/queryPage')")
    public Object queryPage(CourseScheduleTeacherSalaryQueryInfo queryInfo) {
        return succeed(courseScheduleTeacherSalaryService.querySalaries(queryInfo));
    }
}
